
# Copyright (C) 2016 Javier Palacios
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# Version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See full
# GPLv2 for more details (http://www.gnu.org/licenses/gpl-2.0.html).

import ansible_library
import unittest
import json
import os
import shutil
import tempfile
import yaml
import tarfile

from ansible.module_utils.urls import open_url


class ansible_library_test ( unittest.TestCase ) :

    @classmethod
    def setUpClass ( cls ) :
        cls._tempdir = tempfile.mkdtemp()
        roles = yaml.load( open( 'tests/fixtures/roles.yml' ) )
        for role in roles :
            role_name = role.pop('name')
            if not os.path.exists( "%s/%s" % ( cls._tempdir , role_name ) ) :
                os.mkdir( "%s/%s" % ( cls._tempdir , role_name ) )
            file_name = "%s/%s.tar.gz" % ( role_name , role['galaxy_info']['version'] )
            with open( "%s/meta.yml" % cls._tempdir , 'w' ) as fd :
                yaml.dump(role, fd, default_flow_style=False)
            with tarfile.open( "%s/%s" % ( cls._tempdir , file_name ) , mode="w:gz" ) as tar :
                tar.add( "%s/meta.yml" % cls._tempdir , "%s/meta/main.yml" % role_name )
        os.unlink("%s/meta.yml" % cls._tempdir)

    @classmethod
    def tearDownClass ( cls ) :
        shutil.rmtree( cls._tempdir )

    def unpack_roles ( self ) :
        roles = yaml.load( open( 'tests/fixtures/extraroles.yml' ) )
        for role in roles :
            role_name = role.pop('name')
            if not os.path.exists( "%s/%s" % ( self._tempdir , role_name ) ) :
                os.mkdir( "%s/%s" % ( self._tempdir , role_name ) )
            file_name = "%s/%s.tar.gz" % ( role_name , role['galaxy_info']['version'] )
            with open( "%s/meta.yml" % self._tempdir , 'w' ) as fd :
                yaml.dump(role, fd, default_flow_style=False)
            with tarfile.open( "%s/%s" % ( self._tempdir , file_name ) , mode="w:gz" ) as tar :
                tar.add( "%s/meta.yml" % self._tempdir , "%s/meta/main.yml" % role_name )
        os.unlink("%s/meta.yml" % self._tempdir)

    def setUp ( self ) :
        ansible_library.app.appconfig['roles_dir'] = self._tempdir
        ansible_library.app.load_roles()
        self.app = ansible_library.app.test_client()
        self.app.testing = True

    def get ( self , url ) :
        res = self.app.get( "/api/%s" % url )
        self.assertEqual( res.status_code , 200 )
        self.assertEqual( res.content_type , "application/json" )
        return json.loads(res.data)

    def put ( self , url ) :
        res = self.app.put( "/api/%s" % url )
        self.assertEqual( res.status_code , 200 )
        self.assertEqual( res.content_type , "application/json" )
        return json.loads(res.data)

    def test_api_version ( self ) :
        '''Verify API version'''
        data = self.get("")
        self.assertEqual( data['current_version'] , 'v1' )

        galaxy = self.app.get( "https://galaxy.ansible.com/api/" )
        galaxy = json.loads(galaxy.data)
        self.assertEqual( data['current_version'] , galaxy['current_version'] )

    def test_local_role ( self ) :
        '''Get data for local role'''
        data = self.get( "v1/roles/?name=fever" )
        self.assertEqual( data['count'] , 1 )
        result = data['results'][0]
        self.assertEqual( result['name'] , "fever" )

    def test_local_versions ( self ) :
        '''Get versions of local role'''
        data = self.get( "v1/roles/?name=fever" )
        data = self.get( "v1/roles/%s/versions/" % data['results'][0]['id'] )
        self.assertEqual( data['count'] , 2 )
        versions = map( lambda v : v['name'] , data['results'] )
        self.assertIn( "v1.0" , versions )
        self.assertIn( "v1.1" , versions )
        v11 = filter( lambda v : v['name'] == "v1.1" , data['results'] )[0]
        self.assertEqual( v11['download'] , "http://localhost/fever/v1.1.tar.gz" )

    def test_download_local_role ( self ) :
        '''Download role tarball'''
        data = self.get( "v1/roles/1/versions/" )
        self.assertNotEqual( data['results'][0]['download'] , "" )
        role_tarball = self.app.get(data['results'][0]['download'])
        self.assertEqual( role_tarball.status_code , 200 )

    def test_proxied_role ( self ) :
        '''Get public galaxy role if not a local one'''
        data = self.get( "v1/roles/?owner__username=Feverup&name=augeas" )
        self.assertEqual( data['count'] , 1 )
        result = data['results'][0]
        self.assertEqual( result['namespace'] , "Feverup" )
        self.assertEqual( result['name'] , "augeas" )

        galaxy = self.app.get( "https://galaxy.ansible.com/api/v1/roles/" , query_string="?owner__username=Feverup&name=augeas" )
        galaxy = json.loads(galaxy.data)
        self.assertEqual( galaxy['count'] , 1 )
        self.assertDictEqual( data['results'][0] , galaxy['results'][0] )

    def test_proxied_versions ( self ) :
        '''Proxying versions from public galaxy'''
        data = self.get( "v1/roles/?owner__username=Feverup&name=augeas" )
        data = self.get( "v1/roles/%s/versions/" % data['results'][0]['id'] )

        galaxy = self.app.get( "https://galaxy.ansible.com/api/v1/roles/" , query_string="?owner__username=Feverup&name=augeas" )
        galaxy = json.loads(galaxy.data)
        galaxy = self.app.get( "https://galaxy.ansible.com%s" % galaxy['results'][0]['related']['versions'] )
        galaxy = json.loads(galaxy.data)
        self.assertListEqual( data['results'] , galaxy['results'] )

    def test_url_for_proxied_role ( self ) :
        '''Check url returned for proxied roles (regression javiplx/ansible-library#1)'''
        data = self.get( "v1/roles/?owner__username=Feverup&name=augeas" )
        data = self.get( "v1/roles/%s/versions/" % data['results'][0]['id'] )
        self.assertEqual( data['results'][0]['download'] , "" )

    def test_version_field_class ( self ) :
        '''Ensure that versions are returned as strings (regression javiplx/ansible-library#8)'''
        data = self.get( "v1/roles/?name=djangoserver" )
        data = self.get( "v1/roles/%s/versions/" % data['results'][0]['id'] )
        self.assertEqual( data['count'] , 2 )
        for d in data['results'] :
            self.assertIsInstance(d['name'], unicode)

    def test_roles_reload ( self ) :
        '''Reload roles from filesystem'''
        self.unpack_roles()

        data = self.put("reload")
        self.assertEqual( data['msg'] , 'Done' )

        data = self.get( "v1/roles/?name=dnsmasq" )
        self.assertEqual( data['count'] , 1 )





# Copyright (C) 2016 Javier Palacios
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# Version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See full
# GPLv2 for more details (http://www.gnu.org/licenses/gpl-2.0.html).

import flask
import tarfile
import yaml
import operator, itertools
import time
import os
import glob
import logging


class abstract_role ( dict ) :

    def __init__ ( self , content={} ) :
        dict.__init__( self , content )

    def set_url ( self , roles_dir , version ) :
        version['download'] = ""

    def expired ( self ) :
        False

class role ( abstract_role ) :

    def __init__ ( self , id ) :
        abstract_role.__init__( self )
        self['id'] = id

    def set_url ( self , root_url , version ) :
        version['download'] = "%s%s/%s.tar.gz" % ( root_url , self['name'] , version['name'] )

class galaxy_role ( abstract_role ) :

    def __init__ ( self , file_path ) :
        tar = tarfile.open( file_path )
        meta = filter( lambda s : s.endswith('meta/main.yml') ,tar.getnames())
        if len(meta) != 1:
            print "WARNING: '%s' is not an ansible role" % file_path
            return
        data = yaml.load( tar.extractfile(meta[0]) )
        abstract_role.__init__( self , data['galaxy_info'] )
        self['dependencies'] = data['dependencies']
        dir_name , file_name = os.path.split( file_path )
        self['name'] = os.path.basename( dir_name )
        self['version'] = file_name.rpartition('.tar')[0]


class proxied_role ( abstract_role ) :

    def __init__ ( self , galaxy_metadata , ttl ) :
        self.expiration = time.time() + ttl
        abstract_role.__init__( self , galaxy_metadata )

    def expired ( self ) :
        time.time() > self.expiration


class library ( flask.Flask ) :

    def __init__ ( self ) :
        flask.Flask.__init__( self , 'ansible-library' )
        self.roles = []
        self.galaxy = []

    conffile = "/etc/ansible-library.yml"
    appconfig = { 'roles_dir': "/var/lib/galaxy",
                  'logfile': None,
                  'listen': "0.0.0.0",
                  'port': 3333,
                  'ttl': 3600,
                  'daemonize': False,
                  'piddir': None,
                  'debug': False
                  }

    def run ( self , *args, **kwargs ) :
        if os.path.isfile( self.conffile ) :
            localconf = yaml.load( open( self.conffile ) )
            if localconf :
                self.appconfig.update( localconf )
        if self.appconfig['daemonize'] :
            if not self.appconfig['logfile'] :
                print "ERROR: daemonization requires a logfile"
                os.sys.exit(1)
            newpid = os.fork()
            if newpid :
                if self.appconfig['piddir'] :
                    pidfile = os.path.join( self.appconfig['piddir'] , 'ansible-library.pid' )
                    with open( pidfile , 'w' ) as fd :
                        fd.write( "%d\n" % newpid )
                os.sys.exit()
            else :
                os.sys.stdout = open(os.devnull, 'w')
                os.sys.stderr = open(os.devnull, 'w')
        if self.appconfig['logfile'] :
            logger = logging.getLogger('werkzeug')
            logger.addHandler( logging.FileHandler( self.appconfig['logfile'] ) )
            os.sys.stdout = open(os.devnull, 'w')
            os.sys.stderr = open(os.devnull, 'w')
        self.load_roles()
        flask.Flask.run( self , host=self.appconfig['listen'],
                                port=self.appconfig['port'],
                                debug=self.appconfig['debug']
                                )

    def load_roles ( self ) :
        _roles = []
        for file_path in glob.glob( os.path.join( self.appconfig['roles_dir'] , "*/*.tar.gz" ) ) :
            _roles.append( galaxy_role( file_path ) )

        _id = 1
        self.roles = []
        _roles = sorted( _roles , key=operator.itemgetter('name') )
        for k, g in itertools.groupby(_roles, operator.itemgetter('name')):
            _role = role( _id )
            _role.update( g.next() )
            _role['versions'] = [ { 'name': str(_role.pop('version')) } ]
            for r in g :
              _role['versions'].append( { 'name': str(r.pop('version')) } )
            self.roles.append( _role )
            _role['summary_fields'] = { 'dependencies': _role.pop('dependencies'),
                                        'versions': _role.pop('versions')
                                        }
            _id += 1





#!/usr/bin/python

# Copyright (C) 2016 Javier Palacios
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# Version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See full
# GPLv2 for more details (http://www.gnu.org/licenses/gpl-2.0.html).

import ansible_library

if __name__ == "__main__":
    ansible_library.app.run()






# Copyright (C) 2016 Javier Palacios
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# Version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See full
# GPLv2 for more details (http://www.gnu.org/licenses/gpl-2.0.html).

import ansible_library.application
import flask
import os

from ansible.module_utils.urls import open_url

me = { "description": "ansible-library REST API",
       "current_version": "v1",
       "available_versions": {"v1": "/api/v1/"}
       }

app = ansible_library.application.library()

@app.route("/api/")
def api():
    return flask.jsonify(me)

@app.route('/api/reload', methods=['PUT'])
def reload():
    flask.current_app.load_roles()
    return flask.jsonify({'msg': 'Done'})

@app.before_request
def before_request():
    for role in flask.current_app.roles :
        if role.expired() :
            flask.current_app.roles.remove( role )

@app.route("/api/v1/roles/")
def get_roles():
    user = flask.request.args.get('owner__username')
    name = flask.request.args.get('name')
    roles = filter( lambda d : d['name'] == name , flask.current_app.roles )
    if not roles :
        galaxy_url = "https://galaxy.ansible.com%s" % flask.request.full_path
        response = flask.json.load(open_url(galaxy_url))
        results = map( lambda x : application.proxied_role(x, flask.current_app.appconfig['ttl']) , response['results'] )
        flask.current_app.roles.extend( results )
        return flask.jsonify(response)
    resp = { "count": len(roles),
             "cur_page": len(roles),
             "num_pages": len(roles),
             "next": None,
             "previous": None,
             "results": roles
             }
    return flask.jsonify(resp)

@app.route("/api/v1/roles/<int:id>/versions/")
def get_versions(id):
    role = filter( lambda d : d['id'] == id , flask.current_app.roles )
    if len(role) == 0 :
        galaxy_url = "https://galaxy.ansible.com%s" % flask.request.full_path
        return open_url(galaxy_url).read()
    versions = role[0]['summary_fields']['versions']
    role_info = { 'summary_fields': { "role": { "id": role[0]['id'] , "name": role[0]['name'] } } }
    map( lambda d : role[0].set_url( flask.request.url_root , d ) , versions )
    map( lambda d : d.update(role_info) , versions )
    resp = { "count": len(versions),
             "cur_page": len(versions),
             "num_pages": len(versions),
             "next": None,
             "previous": None,
             "results": versions
             }
    return flask.jsonify(resp)

@app.route("/<rolename>/<roleversion>.tar.gz")
def download(rolename, roleversion):
    srcdir = os.path.join( flask.current_app.appconfig['roles_dir'] , rolename )
    return flask.send_from_directory( srcdir , "%s.tar.gz" % roleversion )





#!/usr/bin/python

from distutils.core import setup

setup(
    name = 'ansible-library',
    version = '1.0',
    description = 'Minimal galaxy server to host private roles',
    author = 'Javier Palacios',
    author_email = 'javiplx@gmail.com',
    license = 'GPLv2',
    url = 'https://github.com/javiplx/ansible-library',
    scripts = [ 'ansible-library.py' ],
    packages = [ 'ansible_library' ],
    install_requires=[ 'Werkzeug>=0.9' , 'flask>=0.10' , 'ansible>=1.9.2' , 'pyyaml' ]
    )

